@@ -126,15 +126,12 @@ static int run()
 			{
 				DBG1(DBG_DMN, "signal of type SIGHUP received. Reloading "
 					 "configuration");
-#ifdef STRONGSWAN_CONF
-				if (lib->settings->load_files(lib->settings, STRONGSWAN_CONF,
-											  FALSE))
+				if (lib->settings->load_files(lib->settings, lib->conf, FALSE))
 				{
 					charon->load_loggers(charon, levels, TRUE);
 					lib->plugins->reload(lib->plugins, NULL);
 				}
 				else
-#endif
 				{
 					DBG1(DBG_DMN, "reloading config failed, keeping old");
 				}
