@@ -32,6 +32,8 @@
 #undef PACKAGE_URL
 /* avoid redefintiion of snprintf etc. */
 #define RUBY_DONT_SUBST
+/* undef our _GNU_SOURCE, as it gets redefined by <ruby.h> */
+#undef _GNU_SOURCE
 #include <ruby.h>
 
 static dumm_t *dumm;
