@@ -242,10 +242,13 @@ static void load_entries(private_attr_provider_t *this)
 				{
 					if (family == AF_INET)
 					{	/* IPv4 attributes contain a subnet mask */
-						u_int32_t netmask;
+						u_int32_t netmask = 0;
 
-						mask = 32 - mask;
-						netmask = htonl((0xFFFFFFFF >> mask) << mask);
+						if (mask)
+						{	/* shifting u_int32_t by 32 or more is undefined */
+							mask = 32 - mask;
+							netmask = htonl((0xFFFFFFFF >> mask) << mask);
+						}
 						data = chunk_cat("cc", host->get_address(host),
 										 chunk_from_thing(netmask));
 					}
