@@ -17,7 +17,8 @@
 
 #include "public_key.h"
 
-ENUM(key_type_names, KEY_RSA, KEY_DSA,
+ENUM(key_type_names, KEY_ANY, KEY_DSA,
+	"ANY",
 	"RSA",
 	"ECDSA",
 	"DSA"
@@ -32,13 +33,69 @@ ENUM(signature_scheme_names, SIGN_UNKNOWN, SIGN_ECDSA_521,
 	"RSA_EMSA_PKCS1_SHA256",
 	"RSA_EMSA_PKCS1_SHA384",
 	"RSA_EMSA_PKCS1_SHA512",
+	"ECDSA_WITH_SHA1_DER",
+	"ECDSA_WITH_SHA256_DER",
+	"ECDSA_WITH_SHA384_DER",
+	"ECDSA_WITH_SHA512_DER",
 	"ECDSA_WITH_NULL",
-	"ECDSA_WITH_SHA1",
 	"ECDSA-256",
 	"ECDSA-384",
 	"ECDSA-521",
 );
 
+ENUM(encryption_scheme_names, ENCRYPT_UNKNOWN, ENCRYPT_RSA_OAEP_SHA512,
+	"ENCRYPT_UNKNOWN",
+	"ENCRYPT_RSA_PKCS1",
+	"ENCRYPT_RSA_OAEP_SHA1",
+	"ENCRYPT_RSA_OAEP_SHA224",
+	"ENCRYPT_RSA_OAEP_SHA256",
+	"ENCRYPT_RSA_OAEP_SHA384",
+	"ENCRYPT_RSA_OAEP_SHA512",
+);
+
+/**
+ * See header.
+ */
+bool public_key_equals(public_key_t *this, public_key_t *other)
+{
+	cred_encoding_type_t type;
+	chunk_t a, b;
+
+	if (this == other)
+	{
+		return TRUE;
+	}
+
+	for (type = 0; type < CRED_ENCODING_MAX; type++)
+	{
+		if (this->get_fingerprint(this, type, &a) &&
+			other->get_fingerprint(other, type, &b))
+		{
+			return chunk_equals(a, b);
+		}
+	}
+	return FALSE;
+}
+
+/**
+ * See header.
+ */
+bool public_key_has_fingerprint(public_key_t *public, chunk_t fingerprint)
+{
+	cred_encoding_type_t type;
+	chunk_t current;
+
+	for (type = 0; type < KEYID_MAX; type++)
+	{
+		if (public->get_fingerprint(public, type, &current) &&
+			chunk_equals(current, fingerprint))
+		{
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
 /*
  * Defined in header.
  */
@@ -66,13 +123,13 @@ signature_scheme_t signature_scheme_from_oid(int oid)
 			return SIGN_RSA_EMSA_PKCS1_SHA512;
 		case OID_ECDSA_WITH_SHA1:
 		case OID_EC_PUBLICKEY:
-			return SIGN_ECDSA_WITH_SHA1;
+			return SIGN_ECDSA_WITH_SHA1_DER;
 		case OID_ECDSA_WITH_SHA256:
-			return SIGN_ECDSA_256;
+			return SIGN_ECDSA_WITH_SHA256_DER;
 		case OID_ECDSA_WITH_SHA384:
-			return SIGN_ECDSA_384;
+			return SIGN_ECDSA_WITH_SHA384_DER;
 		case OID_ECDSA_WITH_SHA512:
-			return SIGN_ECDSA_521;
+			return SIGN_ECDSA_WITH_SHA512_DER;
 		default:
 			return SIGN_UNKNOWN;
 	}
