@@ -53,8 +53,7 @@ static int acert()
 			case 'h':
 				goto usage;
 			case 'g':
-				digest = enum_from_name(hash_algorithm_short_names, arg);
-				if (digest == -1)
+				if (!enum_from_name(hash_algorithm_short_names, arg, &digest))
 				{
 					error = "invalid --digest type";
 					goto usage;
@@ -197,6 +196,7 @@ static int acert()
 	}
 	else
 	{
+		set_file_mode(stdin, CERT_ASN1_DER);
 		if (!chunk_from_fd(0, &encoding))
 		{
 			fprintf(stderr, "%s: ", strerror(errno));
@@ -233,6 +233,7 @@ static int acert()
 		error = "encoding attribute certificate failed";
 		goto end;
 	}
+	set_file_mode(stdout, form);
 	if (fwrite(encoding.ptr, encoding.len, 1, stdout) != 1)
 	{
 		error = "writing attribute certificate key failed";
